import matplotlib.pyplot as plt
from numpy import *
from collections import Counter
import numpy as np
import pylab as pl
from matplotlib.colors import ListedColormap
from sklearn import neighbors, datasets, cluster, preprocessing, decomposition, svm, datasets
from sklearn.decomposition import PCA
import pandas as pd
from pandas.tools.plotting import scatter_matrix
import numpy.random as random
from mpl_toolkits.mplot3d import Axes3D
import glob
import os
import time
from IPython.display import display
from IPython.display import Image

import scipy;
from scipy	import stats;
import sklearn;
import sklearn.ensemble;
import sklearn.neighbors
from sklearn.preprocessing import StandardScaler
from sklearn import metrics
from sklearn.metrics import *
from sklearn.metrics import roc_curve, auc
from sklearn.preprocessing import label_binarize
from sklearn.cross_validation import train_test_split
from sklearn.multiclass import OneVsRestClassifier

import os
import subprocess
from sklearn.tree import DecisionTreeClassifier, export_graphviz
from sklearn import tree
from sklearn import datasets
from IPython.display import Image
#import seaborn as sns
from mpl_toolkits.axes_grid1 import make_axes_locatable
import pydotplus
from sklearn.externals.six import StringIO

# -*- coding: utf-8 -*-
def run_cv(X,y,clf_class,printDebug = False , clf=None):
    # Construct a kfolds object
    kf = sklearn.cross_validation.KFold(len(y),n_folds=5,shuffle=True)
    y_pred = y.copy()
    
    # Iterate through folds\
    i = 0;
    for train_index, test_index in kf:
        X_train, X_test = X[train_index], X[test_index]
        y_train = y[train_index]
        # Initialize a classifier with key word arguments
        clf = clf_class(**kwargs) if (clf is None)  else clf;
        if (printDebug): print ("*",i, end ="");
        clf.fit(X_train,y_train)
        y_pred[test_index] = clf.predict(X_test)
        i = i +1;
    if (printDebug): print ("*");
    return y_pred, clf

# -*- coding: utf-8 -*-
def run_cvTT(X,y,clf_class,printDebug = True , clf=None):
    X_train, X_test, y_train, y_test = cross_validation.train_test_split( X, y, test_size=0.2, random_state=0);
            
    
def accuracy(y_true,y_pred):
    # NumPy interprets True and False as 1. and 0.
    return np.mean(y_true == y_pred)

#
# Call:
# cms = [("Decision Tree", [[25,24],[23,22]])]
#
def draw_confusion_matrices(confusion_matricies,class_names):
    class_names = class_names.tolist() if type(class_names) != list else class_names
    fig = plt.figure(figsize = (20,5))
    for i,cm in enumerate(confusion_matricies):
        classifierName, matrix = cm[0], cm[1]
        #cmstr = str(cm)

        ax = fig.add_subplot(1,8,i+1)
        plt.subplots_adjust(wspace = .4);
        cax = ax.matshow(matrix, cmap='seismic', interpolation='nearest')
        #plt.title('CM: %s' % classifier + "\n" + cmstr)
        plt.title(classifierName)
        plt.grid(None)
        if (i ==0 ):
            i=0;
            #fig.colorbar(cax);
            
            
        ax.set_xticklabels([''] + class_names)
        ax.set_yticklabels([''] + class_names)
        plt.xlabel('Predicted')
        plt.ylabel('True')
        for (ii, jj), z in np.ndenumerate(matrix):
            ax.text(jj, ii, '{:0.1f}'.format(z), ha='center', va='center',
                    bbox=dict(facecolor='white', edgecolor='0.3'))
    plt.show()


##
# Draw most 15 significant 
#
def DrawFeatureImportance(dft,clf, title="", ax =None, m=15):
    if ( not hasattr(clf,'feature_importances_') ):
        print ("No Feature Importance matrix for this classifier:", clf)
        return;
        
    # Get Feature Importance from the classifier
    feature_importance = clf.feature_importances_
    # Normalize The Features
    feature_importance = 100.0 * (feature_importance / feature_importance.max())
    sorted_idx = np.argsort(feature_importance)
    sorted_idx10 = sorted_idx[-m:]; #[0:5]  # TODO: Print Top 10 only ??
    pos = np.arange(sorted_idx10.shape[0]) + .5
    fc10=np.asanyarray(dft.columns.tolist())[sorted_idx10];
    
    if ( ax == None):
        plt.figure(figsize=(5, 3));
    plt.barh(pos, feature_importance[sorted_idx10], align='center', color='#7A68A6')
    plt.yticks(pos, fc10)
    plt.xlabel('Relative: '+ title)
    plt.title('Variable Importance')
    if ( ax == None): plt.show()

def DrawFeatureImportanceMatrix(df, clfs):
    fig = plt.figure(figsize = (25,3))
    plt.subplots_adjust(wspace = 1.2);
    for i in range(int (len(clfs)/2)) :
        classifierName, clf = clfs[i*2], clfs[i*2+1]
        if ( not hasattr(clf,'feature_importances_') ):
            continue;
        #cmstr = str(cm)
        #print classifierName;
        plt.subplots_adjust(wspace = 1.4);
        ax = fig.add_subplot(1,8,i+1)
        DrawFeatureImportance(df, clf, classifierName, ax)
        #plt.title(classifierName)
            
    plt.show()
    
# This function will take the categorical value or any column and
# either replaces inline or create a new column with enumeration of 
# Values for a left column will be changed as shown to the right array:
# For ex. [a,a,a,b,b,c,c,c,c,d, d] => [0,0,0,1,1,2,2,2,2,3, 3]
#
def encodeCategorical(df, columnName, newColumnName = None, makeCopy = False):
    df_mod = df.copy() if makeCopy else df;
    targets = df_mod[columnName].unique()
    mapToInt = {name: n for n, name in enumerate(targets)}
    newColumnName = newColumnName if (newColumnName!=None) else columnName;
    df_mod[newColumnName] = df_mod[columnName].replace(mapToInt)

    return (df_mod, targets, mapToInt)
    
#Classification Problems 
# Usually in case of classifcation, it is best to draw scatter plot of 
# Target Varible using Scatter plot
# df, t,m = encodeCategorical(dfL, "FiveTile1", "Target" );
# scatter_matrix(dfL, alpha=1, figsize=(10,10), s=100, c=df.Target);
# print "categorical Plot {}".format(m)
#
#
# Df - Data Frame that does not have a Predict Column
# y  - Predict Column Series 
def Classify(df, y, 
             printDebug = True ,
             drawConfusionMatrix = True,
             classifiers = None,
             scale =True
             ):
    if ( df is None or y is None):
        raise Exception("No Data Given");

    t = df.select_dtypes(exclude=[np.number])
    if ( len(t.columns) > 0) :
        raise Exception("nonnumeric columns? "  + t.columns);

    l = preprocessing.LabelEncoder()
    class_names = y.unique()
    y=l.fit_transform(y);

    df.fillna(0, inplace=True)
    X = df.as_matrix().astype(np.float)

    if (scale):
        scaler = StandardScaler()
        X = scaler.fit_transform(X)
      
    print ("Feature space holds %d observations and %d features" % X.shape)
    print ("Unique target labels:", class_names)
         
    cls = [# Note: SVM takes long time to run - so commenting out
           #"SVM"               , sklearn.svm.SVC(), 
           "Random Forest"     , sklearn.ensemble.RandomForestClassifier(),
           #"K-NN"              , sklearn.neighbors.KNeighborsClassifier(),
           "DecisionTree Gini" , DecisionTreeClassifier(max_depth=4, criterion="gini"), 
           "DecisionTree Entr" , DecisionTreeClassifier(max_depth=4, criterion="entropy"), 
           "Gradient Boosting" , sklearn.ensemble.RandomForestClassifier(),
           #"Logit Regression"  , sklearn.linear_model.LogisticRegression()
           ];

    if (not classifiers is None ):
        cls = classifiers;

    y_preds = {}
    ret_accuracy = [];
    cms = [];
    clfs = {}
    for i in arange( int (len(cls)/2) ):
        nm = cls[i*2];
        cl = cls[i*2 +1]
        y_pred, clfi = run_cv(X,y, None, clf=cl, printDebug=printDebug)
        y_preds[nm] = y_pred
        clfs[nm] = clfi
        ac  = accuracy(y, y_pred);
        cm = confusion_matrix(y, y_pred )
        ret_accuracy.append( (nm, ac, cm) )
        if (printDebug): 
            print ("%20s accuracy: %03f "% (nm, ac) );
            #print('{}\n'.format(metrics.classification_report(y, y_pred)))
            #print("%20s r^2 score: %03f"% (nm,sklearn.metrics.r2_score(y, y_pred, sample_weight=None, multioutput=None)))
            print("%20s r^2 score: %03f"% (nm,sklearn.metrics.r2_score(y, y_pred, sample_weight=None)))
        cms.append( (nm, cm) );
    if (drawConfusionMatrix): 
        #print cms, class_names
        draw_confusion_matrices(cms, class_names);
        DrawFeatureImportanceMatrix(df, cls)

    return (X, y, ret_accuracy,cls, y_preds,clfs);

def visualizeTree(dcls, feature_names, class_names= None):
    dot_data = StringIO()  
    tree.export_graphviz(dcls, out_file=dot_data,  
                         feature_names= feature_names,  
                         class_names= class_names,  
                         filled=True, rounded=True,  
                         special_characters=True)  
    graph = pydotplus.graph_from_dot_data(dot_data.getvalue())
    display(Image(graph.create_png()))
    return "";

##======================= DRAW Decision Trees here 
def DrawDecisionTree(X,y, cls, class_names=None):
    imgs=[]
    
    if ( class_names is None):
        class_names = y.unique().astype(str);
        class_names.sort()
    if (str(type(cls)).find('DecisionTreeClassifier') > 0 ):
        visualizeTree(cls, X.columns, class_names=class_names)
    else:
        for k in range(int( len(cls)/2) ) :
            dcls = cls[k*2+1];
            if (str(type(dcls)).find('DecisionTreeClassifier') > 0):
                visualizeTree(dcls, X.columns, class_names=class_names)

    
#======================== Get COde For DecisionTree        
# Here is how to use this code
#if __name__ == '__main__':
#    print("\n-- get data:")
#    df = dfL;
#
#    print("\n-- df.head():")
#    print(df.head(), end="\n\n")
#
#    features = ["SepalLength", "SepalWidth", "PetalLength", "PetalWidth"]
#    df, targets,mi = encodeCategorical(df, "Name", "Target")
#    y = df["Target"]
#    X = df[features]
#
#    dt = DecisionTreeClassifier(min_samples_split=20, random_state=99)
#    dt.fit(X, y)
#
#    print("\n-- get_code:")
##    get_code(dt, features, targets)
#
#    print("\n-- look back at original data using pandas")
#    print("-- df[df['PetalLength'] <= 2.45]]['Name'].unique(): ",
#          df[df['PetalLength'] <= 2.45]['Name'].unique(), end="\n\n")
#
#    visualizeTree(dt, features)

def getCodeOfDecisionTree(tree, feature_names, target_names, spacer_base="    "):
    """Produce psuedo-code for decision tree.
    
    Args
    ----
    tree -- scikit-leant DescisionTree.
    feature_names -- list of feature names.
    target_names -- list of target (class) names.
    spacer_base -- used for spacing code (default: "    ").

    Notes
    -----
    based on http://stackoverflow.com/a/30104792.
    """
    left      = tree.tree_.children_left
    right     = tree.tree_.children_right
    threshold = tree.tree_.threshold
    features  = [feature_names[i] for i in tree.tree_.feature]
    value = tree.tree_.value
   
    def recurse(left, right, threshold, features, node, depth):
        spacer = spacer_base * depth
        if (threshold[node] != -2):
            print(spacer + "if ( " + features[node] + " <= " + \
                  str(threshold[node]) + " ) {")
            if left[node] != -1:
                    recurse (left, right, threshold, features, left[node],
                            depth+1)
            print(spacer + "}\n" + spacer +"else {")
            if right[node] != -1:
                    recurse (left, right, threshold, features, right[node],
                             depth+1)
            print(spacer + "}")
        else:
            target = value[node]
            for i, v in zip(np.nonzero(target)[1], target[np.nonzero(target)]):
                target_name = target_names[i]
                target_count = int(v)
                print(spacer + "return " + str(target_name) + " ( " + \
                      str(target_count) + " examples )")
    
    recurse(left, right, threshold, features, 0, 0)





import numpy as np
from sklearn import preprocessing
from sklearn.decomposition import PCA
import pandas as pd
import os
from IPython.display import display
from IPython.display import HTML
import dateutil;
import json;
import urllib;

import matplotlib
matplotlib.style.use('ggplot')

def computePCA(d, components = 2, columnNames=None):

    if (d.shape[1] < 2) :
        print ("Number of components are already less than 2")

    pca = PCA(n_components= components)
    pca.fit(d)

    #usorted=  zip(df.columns.values, pca.components_[0])
    #sort = sorted (usorted, key = lambda x:x[1])
    #print (sort)

    return pca
#======================================================================
'''
Evaluating the cost of the clusters
'''
'''
The costOfCluster is the cost assigned to a k-means cluster.
K-means algorithm optimizes on the cost function SUM( ||xi - Mj||^2) - where Mj is the
centroid of of cluster. In this case, if xi ( i is from 1..m) is assigned to cluster j ( j from 1..k)
then the ||xi - Mj||^2) is the square of the distaance
'''
#from scipy.spatial.distance import *
#from sklearn import metrics
#from sklearn.metrics import pairwise_distances

def costOfCluster(kmeans, data) :
    l = kmeans.labels_
    c = kmeans.cluster_centers_
    s = 0.0
    for i, x in enumerate(data):
        c1 = euclidean (x, c[l[i]]);
        s =  s + c1 ** 2
    return s;

# x must be an np array. Thi
def elbowIndex(x):
    if ( x.shape[0] <= 2):
        return 0
    d = 0
    d = x[:-1] - x[1:]
    second_d = np.abs(d[:-1] - d[1:])
    if (second_d.shape[0] <=0):
        return 0;
    print ("+++", second_d.shape)
    return 1 + np.argmax(second_d)



import matplotlib.pyplot as plt
from numpy import *
from collections import Counter
import numpy as np
import pylab as pl
import pandas as pd
from pandas.tools.plotting import scatter_matrix
import numpy.random as random
from mpl_toolkits.mplot3d import Axes3D
from sklearn.feature_extraction import DictVectorizer
import time
import datetime
from sklearn.preprocessing import StandardScaler
from sklearn import neighbors, datasets, cluster, preprocessing, decomposition
  
def ConvertCaegoricalToBool(df, n):
    p = df[n];
    
    yy=p.astype(str)
    unique=np.unique(yy)

    if ( len(unique) > 2 ):
        raise Exception("supports only binary - Found (%d) categories!" % len(unique))
    unique.sort();
    y= np.where(yy==unique[0], int(0), int(1) )
    return y;
#
# This function will take the categorical value or any column and
# either replaces inline or create a new column with enumeration of 
# Values for a left column will be changed as shown to the right array:
# For ex. [a,a,a,b,b,c,c,c,c,d, d] => [0,0,0,1,1,2,2,2,2,3, 3]
#
# pd.Categorical(['a', 'a', 'a', 'b', 'b', 'c']).codes
# pd.dummies    

#def encodeCategorical(df, columnName, newColumnName = None, makeCopy = False):
#    print "====> encodeCategorical ", makeCopy;
#    df_mod = df.copy() if makeCopy else df;
#    targets = df_mod[columnName].unique()
#    targets.sort();
#    mapToInt = {name: n for n, name in enumerate(targets)}
#    newColumnName = newColumnName if (newColumnName!=None) else columnName;
#    df_mod.loc[:,newColumnName] = df_mod[columnName].replace(mapToInt)#
#
#   df_mod[columnName].replace(mapToInt, inplace=True)
#
#    return (df_mod, targets, mapToInt)
#
    
#def encodeCategorical(df, columnName = None):
#    l = preprocessing.LabelEncoder()
#    y = df[columnName] if columnName else df;
#    l.fit_transform(dfL[k])
#    return y;

#
# This calls for get_dummies
#
def vectorize(df, cols):
    vec = DictVectorizer(dtype=float32)
    mkdict = lambda row: dict((col, row[col]) for col in cols)
    vecData = pd.DataFrame(vec.fit_transform(df[cols].apply(mkdict, axis=1)).toarray())
    vecData.columns = vec.get_feature_names()
    vecData.index = df.index

    df = df.drop(cols, axis=1)
    df = df.join(vecData)

    return (df, vecData, vec)

def ToUNIXTime(dfi, columnName = None):
    y = dfi if columnName is None else dfi[columnName];
    Lam=lambda k: int((k-datetime.datetime(1970,1,1)).total_seconds())   
    r = y.apply(Lam)

    return r;

def ToYYYYTime(dfi, columnName = None):
    Lam=lambda k: int(datetime64(k).item().strftime("%Y%m%d%H%M%S"))  
    df = dfi if type(dfi) == pd.core.series.Series else dfi[columnName];
    df.fillna(0, inplace=True)
    r = df.apply(Lam)

    return r;

def FromUNIXTime(dfi, columnName):
    l=lambda k:  np.datetime64(int(k), "ms") if (k > 1000000000000) else np.datetime64(int(k), "s")
    #newColumnName = newColumnName if (newColumnName!=None) else columnName;
    #dfi.loc[:,newColumnName] = dfi[columnName].apply(l);
    r = dfi[columnName].apply(l);

    return r

#this counts upto count unique values, if it exceeds the count, it returns None
def myUnique(y, threshhold =50):
    m=set()
    for i in y:
        m.add(i)
        if (len(m) > threshhold):
            return None;
    return m
    
# 
# Category = True - instead suse Vectoriser 
#
def prepareDF(dfi, makeCopy = False, 
              threshHold = 50, fillna=0,
              category=True):
    df = dfi.copy() if makeCopy else dfi;

    df.fillna(fillna, inplace=True)
    t = df.select_dtypes(exclude=[np.number])
    
    if ( len(t.columns) <= 0) :
        return df;

    l = preprocessing.LabelEncoder()
    
    n = df.shape[0]    # number of rows
    for k in t.columns:
        if (len(k) <= 0):
            continue;
        if ( df[k].dtype == 'bool'):
            df[k] = df[k].astype('int');
            continue;
        if (df[k].dtype == np.dtype('datetime64[ns]')):
            #print "YEY";
            #df[k] = ToUNIXTime(df,k);
            df[k] = ToYYYYTime(df,k);
            continue;
        
        
        targets = df[k].unique()
        #targets.sort()
        #mapToInt = {name: n for n, name in enumerate(targets)}
        o = len(targets)
        r = float(o)/n*100;
        if (threshHold == None or threshHold ==0 or (o < threshHold or r < 10)) :
            if ( o == 2 or category ):
                if ( o < threshHold or r < 10 ):
                   try:
                      #print ("processing ", k);
                      df[k] = l.fit_transform(df[k])
                   except:
                      print ("Error while processing {} {} ".format(k,df.columns[k]))
            else:
                (df,v1,v2) = vectorize(df,[k])
    
    t = df.select_dtypes(exclude=[np.number])
    df = df.drop(t, axis=1)
    print ("Dropping in prepareDF - ", t.columns);

    return df




from __future__ import absolute_import

"""Functions to quickly display charts in a Notebook.
"""

import string
import random
import json
import copy
import pandas as pd

from IPython.core import getipython
from IPython.core.display import display, HTML
import numpy as np;

# Note that Highstock includes all Highcharts features.
HIGHCHARTS_SCRIPTS1 = "https://code.highcharts.com/highcharts.js"
HIGHCHARTS_SCRIPTS2 = """
<script src="https://code.highcharts.com/stock/highstock.js"></script>
<script src="https://code.highcharts.com/highcharts-more.js"></script>
<script src="https://code.highcharts.com/modules/exporting.js"></script>
<script src="https://code.highcharts.com/modules/histogram-bellcurve.js"></script>
"""

def load_highcharts():
    #display(HTML(HIGHCHARTS_SCRIPTS1))
    display(HTML(HIGHCHARTS_SCRIPTS2))

# Automatically insert the script tag into your Notebook.
# Call when you import this module.
if 'IPKernelApp' in getipython.get_ipython().config:
    load_highcharts()


class HighCharts:
    TS='''<script>
Highcharts.chart('CHART_DIV', {
    navigation: { buttonOptions: { enabled: false } },
    credits: { enabled: false},
    chart: { type: 'line' ,  zoomType: 'x' },
    title: { text: 'CHART_TITLE' , zoomType:'xy' },
    subtitle: { text: 'CHART_SUB_TITLE' },
    xAxis: { title: { text: 'CHART_X_AXIS_TITLE' }, type: 'datetime'},
    yAxis: { title: { text: 'CHART_Y_AXIS_TITLE'}},
    plotOptions: {
        scatter: {
            marker: { radius: 3},
            states: {
                hover: {
                    enabled: true,
                    lineColor: 'rgb(100,100,100)'
                }
            }
        },
        line: {
            animation: true,
            dataLabels: {  enabled: false  },
            enableMouseTracking: true,
            lineWidth: 0.1,
            marker: { radius: 3} 
            },
        histogram: {
            type: 'column',
            },
            
        series: { point: { events: { click: 'CLICK_FUNCTION' } } }
        },
        DATA
});
</script>
'''
    
    @staticmethod
    def tsDF(df, x=None, cols=[], names=[], div=None, title='', subtitle='', yTitle='',xTitle='', num=1000000,
              onClick='function(){g=this;console.log(g.index, g.y, g.x)}', ctype=None, dtype='int'):
        '''
        Plot High Chart assuing the data is in panadas dataframe. 
        Provide a div if it needs to be plotted inside a div or one will be creaeted for you. 
        '''
        
        ts =HighCharts.TS;
        if ( div is None ):
            div = 'DIVchart_' + str(np.random.randint(1000000))
            ts = '<div id="{}" style="height:200px"></div>\n'.format(div) + ts

        if (type(x) == str):   xa = df[x]
        elif (x is None): xa = pd.Series(df.index)
        elif (type(x) == int): xa = df[df.columns[x]]
        else: xa = x

        dt = xa
        try:
            if (xa.astype(int)[0] > 1000000000000000000):
                dt=(xa.astype(int)/1000000)[0:num]
            elif (xa.astype(int)[0] > 1000000000000000):
                dt=(xa.astype(int)/1000)[0:num]
        except:
            pass;

        dt = [int(c) for c in dt]
        dd=[]

        for i, c in enumerate(cols):
            if ( type(c) == str):
                d=list(df[c].values[0:num])
                cn=c
            elif( type(c) == int ):
                cn=df.columns[c]
                d=list(df[cn].values[0:num])
            else:
                cn= "List "+i if names is None or i >= len(names) else names[i]
                d=c;

            dat=list(zip(dt, d))
            dd.append({'name': cn, 'data': dat })

        s = 'series: ' + pd.io.json.dumps(dd) +""
        ts= ts.replace('DATA',s)
        ts=ts.replace('CHART_DIV', div)
        ts=ts.replace('CHART_TITLE', title)
        ts=ts.replace('CHART_SUB_TITLE', subtitle)
        ts=ts.replace('CHART_Y_AXIS_TITLE', yTitle)
        ts=ts.replace('CHART_X_AXIS_TITLE', xTitle)
        ts=ts.replace("'CLICK_FUNCTION'", onClick)
        ts=ts.replace('datetime', dtype)

        if (ctype != None):
            ts=ts.replace("type: 'line'","type: '{}'".format(ctype));

        display(HTML(ts))    
        return ts;
    
    
    histHTML ='''
<script>
var data2=[[140, 1],[150, 9],[160, 15],[170, 15]]
//DATA
Highcharts.chart('container', {
    navigation: { buttonOptions: { enabled: false } },
    credits: { enabled: false},
    title: {text: 'CHART-TITLE'},
    xAxis: [{title: { text: 'X-AXIS-TITLE' }}, {title: { text: '' }}],
    yAxis: [{title: { text: '' }}, {title: { text: 'Y-AXIS-TITLE' }}],

    series: [{
        name: 'H',
        type: 'histogram',
        data: data,
        showInLegend: false, 
        xAxis: 1,
        yAxis: 1,
        baseSeries: 0,
        pointPlacement: 'between',
        binsNumber: 10
    }
    ]
});
</script>
'''
    @staticmethod
    def hist(s, bins='auto', div=None, name='', title='', yTitle='',xTitle=''):
        html = HighCharts.histHTML
        if ( div is None ):
            div = 'DIVchart_' + str(np.random.randint(1000000))
            html += '<div id="{}" style="height:200px"></div>\n'.format(div)  
            
        x,y = np.histogram(s, bins=bins)
        z=zip(y,x)
        d=str(list(z)).replace('(','[').replace(')',']')
        html = html.replace("//DATA", "data="+str(d))
        html = html.replace("CHART-TITLE", title)
        html = html.replace('X-AXIS-TITLE', xTitle)
        html = html.replace('Y-AXIS-TITLE', yTitle)
        html = html.replace('container', div)
        display(HTML(html))
